#!/usr/bin/env python

from MoinMessage import *
from email.mime.text import MIMEText
from email.parser import Parser
import sys

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

def decrypt(message):

    "Decrypt 'message' if necessary."

    if is_encrypted(message):
        text = gpg.decryptMessage(message)
        return Parser().parse(StringIO(text))
    else:
        return message

def verify(message):

    "Verify 'message' if possible."

    if is_signed(message):
        return gpg.verifyMessage(message)
    else:
        return None, None, message

if __name__ == "__main__":
    try:
        signer = sys.argv[1]
        recipient = sys.argv[2]
        url = sys.argv[3] + "?action=FetchMessages"
        args = sys.argv[4:]
    except IndexError:
        args = None

    if not args:
        print >>sys.stderr, "Need a signer, recipient, URL and some commands as arguments to this program."
        sys.exit(1)

    message = Message()
    parts = []

    part = MIMEText("\n".join(args), "plain", sys.stdin.encoding)
    message.add_update(part)

    email_message = message.get_payload()
    gpg = GPG()

    try:
        signed_message = gpg.signMessage(email_message, signer)
        encrypted_message = gpg.encryptMessage(signed_message, recipient)
        text = sendMessage(encrypted_message, url, "PUT")
        mailbox = Parser().parse(StringIO(text))

        mailbox = decrypt(mailbox)
        fingerprint, identity, content = verify(mailbox)

        if fingerprint and identity:
            print >>sys.stderr, "Messages signed by", fingerprint, identity
        else:
            print >>sys.stderr, "Message not signed!"

        print >>sys.stderr

        if content.is_multipart():
            for part in content.get_payload():
                part = decrypt(part)
                fingerprint, identity, part = verify(part)
                if fingerprint and identity:
                    print >>sys.stderr, "Part signed by", fingerprint, identity
                print part
        else:
            print content

    except MoinMessageError, exc:
        print >>sys.stderr, exc

# vim: tabstop=4 expandtab shiftwidth=4
